div.osmfield-wrapper {
	position: relative;
	display: inline;
}

div.osmfield-map {
	width: 350px;
	height: 250px;
	border: 1px solid black;
	position: absolute;
	left: 0;
	top: 30px;
	z-index: 1000;
}

